@@ -594,7 +594,7 @@ fi
 if [ $mtest = 0 ]; then
     mfile=`cat $tmp`
     for z in $mfile ; do
-	cfgline="$cfgline --with-man-dir=$z"
+	cfgline="$cfgline --with-mandir=$z"
     done
 fi
 
