@@ -29,7 +29,7 @@
 enum {
   SH_ERR_T_START  = 0,
 
-  /* 1-9 = SH_LEVEL_XXX */
+  /* 1-13 = SH_LEVEL_XXX */
 
   SH_ERR_T_RO      = SH_LEVEL_READONLY,
   SH_ERR_T_LOGS    = SH_LEVEL_LOGFILES,
@@ -39,13 +39,16 @@ enum {
   SH_ERR_T_ATTR    = SH_LEVEL_ATTRIBUTES,  
   SH_ERR_T_USER0   = SH_LEVEL_USER0,  
   SH_ERR_T_USER1   = SH_LEVEL_USER1,  
+  SH_ERR_T_USER2   = SH_LEVEL_USER2,  
+  SH_ERR_T_USER3   = SH_LEVEL_USER3,  
+  SH_ERR_T_USER4   = SH_LEVEL_USER4,  
   SH_ERR_T_PRELINK = SH_LEVEL_PRELINK,  
 
-  SH_ERR_T_DIR    = 10,
-  SH_ERR_T_FILE   = 11,
-  SH_ERR_T_NAME   = 12,
+  SH_ERR_T_DIR    = 13,
+  SH_ERR_T_FILE   = 14,
+  SH_ERR_T_NAME   = 15,
 
-  SH_ERR_T_END    = 13
+  SH_ERR_T_END    = 16
 };
 
 
@@ -80,25 +83,25 @@ extern int  ShDFLevel[SH_ERR_T_END];
 
 /* set mask for message class
  */
-int sh_error_log_mask (char * c);
-int sh_error_print_mask (char * c);
-int sh_error_mail_mask (char * c);
-int sh_error_export_mask (char * c);
-int sh_error_syslog_mask (char * c);
-int sh_error_external_mask (char * c);
-int sh_error_database_mask (char * c);
-int sh_error_prelude_mask (char * c);
+int sh_error_log_mask (const char * c);
+int sh_error_print_mask (const char * c);
+int sh_error_mail_mask (const char * c);
+int sh_error_export_mask (const char * c);
+int sh_error_syslog_mask (const char * c);
+int sh_error_external_mask (const char * c);
+int sh_error_database_mask (const char * c);
+int sh_error_prelude_mask (const char * c);
 
 
-int sh_error_verify (char * s);
-int sh_error_logverify_mod (char * s); /* just list, don't verify */
-int sh_error_logverify (char * s);
+int sh_error_verify (const char * s);
+int sh_error_logverify_mod (const char * s); /* just list, don't verify */
+int sh_error_logverify (const char * s);
 
 void sh_error_dbg_switch(void);
 
 #ifdef SH_WITH_SERVER
 void sh_error_set_peer(const char * str);
-int  set_flag_sep_log (char * str);
+int  set_flag_sep_log (const char * str);
 #endif
 
 /* init or re-init log facilities that need it
@@ -107,7 +110,7 @@ void sh_error_fixup();
 
 /* convert a string to a numeric priority
  */ 
-int sh_error_convert_level (char * str_s);
+int sh_error_convert_level (const char * str_s);
 
 /* only to stderr (GOOD/BAD)
  */
@@ -115,52 +118,52 @@ void sh_error_only_stderr (int flag);
 
 /* set syslog facility 
  */
-int  sh_log_set_facility (char * c);
+int  sh_log_set_facility (const char * c);
 
 /* define message header
  */
-int sh_error_ehead (/*@null@*/char * s);
+int sh_error_ehead (/*@null@*/const char * s);
 
 /* set level for error logging 
  */
-int sh_error_setlog(char * str_s);
+int sh_error_setlog(const char * str_s);
 
 /* set severity levels
  */
-int sh_error_set_iv (int iv, char *  severity_s);
+int sh_error_set_iv (int iv, const char *  severity_s);
 
 /* set priorities
  */
-int sh_error_set_level(char * str_s, int *facility);
+int sh_error_set_level(const char * str_s, int *facility);
 
 /* set level for TCP export
  */
-int sh_error_setexport(char *  str_s);
+int sh_error_setexport(const char *  str_s);
 
 /* set level for syslog
  */
-int sh_error_set_syslog (char * flag_s);
+int sh_error_set_syslog (const char * flag_s);
 
 /* set level for printing
  */
-int sh_error_setprint(char *  flag_s);
+int sh_error_setprint(const char *  flag_s);
 
 /* set severity for external
  */
-int sh_error_set_external (char * str_s);
+int sh_error_set_external (const char * str_s);
 
 /* set severity for external
  */
-int sh_error_set_database (char * str_s);
+int sh_error_set_database (const char * str_s);
 
 /* set severity for external
  */
-int sh_error_set_prelude (char * str_s);
+int sh_error_set_prelude (const char * str_s);
 
 
 /* set level for mailing
  */
-int sh_error_setseverity (char * flag);
+int sh_error_setseverity (const char * flag);
 
 /* set debug level
  */
@@ -177,7 +180,7 @@ void sh_error_logrestore(void);
 
 /* (re)set the console device(s)
  */
-int sh_log_set_console (char * address);
+int sh_log_set_console (const char * address);
 void reset_count_dev_console(void);
 
 #ifdef WITH_MESSAGE_QUEUE
@@ -187,7 +190,7 @@ void close_ipc (void);
 
 /* enable message queue
  */
-int enable_msgq(char * foo);
+int enable_msgq(const char * foo);
 #endif
  
 #endif
