@@ -381,7 +381,7 @@ void sh_init (void)
   /* The local host.
    */
   (void) sl_strlcpy (sh.host.name,  _("localhost"),  SH_MINIBUF);
-  sh.host.system[0]     = '\0';
+  sh.host.system[0]     = '\0'; /* flawfinder: ignore *//* ff bug */
   sh.host.release[0]    = '\0';
   sh.host.machine[0]    = '\0';
 
@@ -581,7 +581,7 @@ void sh_init (void)
   skey->sigkey_new[0]   = '\0';
   skey->mailkey_old[0]  = '\0';
   skey->mailkey_new[0]  = '\0';
-  skey->crypt[0]        = '\0';
+  skey->crypt[0]        = '\0'; /* flawfinder: ignore *//* ff bug */
   skey->session[0]      = '\0';
   skey->vernam[0]       = '\0';
 
@@ -724,16 +724,25 @@ static pid_t * procdirSamhain ()
   char       * tail;
   char         exef[128];
 
+  if (!pidlist)
+    return NULL;
+
   for (i = 0; i < 65535; ++i) pidlist[i] = 0;
   i = 0;
 
   if (0 != stat(SH_INSTALL_PATH, &buf))
-    return NULL;
+    {
+      free(pidlist);
+      return NULL;
+    }
 
   ino = (long) buf.st_ino;
     
   if (NULL == (dp = opendir("/proc")))
-    return NULL;
+    {
+      free(pidlist);
+      return NULL;
+    }
   while (NULL != (d = readdir(dp)) && i < 65535)
     {
       if (0 != strcmp(d->d_name, ".") && 0 != strcmp(d->d_name, ".."))
@@ -807,7 +816,7 @@ static pid_t pidofSamhain (int flag)
 
   if (!fp)
     { if (errno != ENOENT) perror(_("fopen")); return 0; }
-  if (NULL == fgets(line, 255, fp))
+  if (NULL == fgets(line, sizeof(line), fp))
     { perror(_("fgets")); (void) fclose(fp); return 0; }
   (void) fclose(fp); 
   p = line; 
@@ -859,18 +868,13 @@ static int samhainctl(int ctl, int * argc, char * argv[])
   int         i;
 
 
-  fullpath = malloc(strlen(SH_INSTALL_PATH)+1);
+  fullpath = strdup (SH_INSTALL_PATH);
   if (fullpath == NULL)
-    { perror(_("malloc")); exit (1); }
-  else
-    strcpy(fullpath, SH_INSTALL_PATH);                 /* known to fit  */
+    { perror(_("strdup")); exit (1); }
 
-  argp[0] = malloc(strlen(SH_INSTALL_PATH)+1);
+  argp[0]  = strdup (SH_INSTALL_PATH);
   if (argp[0] == NULL)
-    { perror(_("malloc")); exit (1); }
-  else
-    strcpy(argp[0], SH_INSTALL_PATH);                  /* known to fit  */
-
+    { perror(_("strdup")); exit (1); }
 
   for (times = 1; times < 32; ++times)  argp[times] = NULL;
 
@@ -878,11 +882,9 @@ static int samhainctl(int ctl, int * argc, char * argv[])
 
   for (times = 2; times < res; ++times)  
     {
-      argp[times-1] = malloc(strlen(argv[times])+1);
+      argp[times-1] = strdup (argv[times]);
       if (argp[times-1] == NULL)
-	{ perror(_("malloc")); exit (1); }
-      else
-	strcpy(argp[times-1], argv[times]);              /* known to fit  */
+	{ perror(_("strdup")); exit (1); }
     }
 
   if (ctl == 1)
@@ -902,7 +904,7 @@ static int samhainctl(int ctl, int * argc, char * argv[])
 	  {
 	    _exit(4);
 	  }
-	(void) execv(fullpath, argp); 
+	(void) execv(fullpath, argp); /* flawfinder: ignore *//* wtf? */
 	if (errno == EPERM)
 	  _exit(4);
 	else if (errno == ENOENT)
@@ -940,7 +942,10 @@ static int samhainctl(int ctl, int * argc, char * argv[])
     {
       pidlist = procdirSamhain ();
       if (pid == 0 && NULL == pidlist) /* pid file not found */ 
-	return (0);
+	{
+	  free(fullpath);
+	  return (0);
+	}
 	  
       status = 0;
       if (pid != 0)
@@ -955,6 +960,7 @@ static int samhainctl(int ctl, int * argc, char * argv[])
 	      ++i;
 	    }
 	}
+      free(fullpath);
       if (status == 7)
 	return 0;
       else
@@ -991,7 +997,7 @@ static int samhainctl(int ctl, int * argc, char * argv[])
 	    exit (1);
 	}
     }
-
+  free(fullpath); /* silence smatch false positive */
   exit (1); /* no exit handler installed yet */
   /*@notreached@*/
   return (0);
@@ -1021,7 +1027,7 @@ static sh_schedule_t *  free_sched (sh_schedule_t * isched)
 
 /* Add a new schedule to the linked list of schedules
  */
-static sh_schedule_t * sh_set_schedule_int (char * str, 
+static sh_schedule_t * sh_set_schedule_int (const char * str, 
 					    sh_schedule_t * FileSchedIn, 
 					    /*@out@*/ int * status)
 {
@@ -1051,7 +1057,7 @@ static sh_schedule_t * sh_set_schedule_int (char * str,
 
 /* Add a new schedule to the linked list FileSchedOne
  */
-int sh_set_schedule_one (char * str)
+int sh_set_schedule_one (const char * str)
 {
   int status;
   FileSchedOne = sh_set_schedule_int (str, FileSchedOne, &status);
@@ -1060,7 +1066,7 @@ int sh_set_schedule_one (char * str)
 
 /* Add a new schedule to the linked list FileSchedTwo
  */
-int sh_set_schedule_two (char * str)
+int sh_set_schedule_two (const char * str)
 {
   int status;
   FileSchedTwo = sh_set_schedule_int (str, FileSchedTwo, &status);
@@ -1074,7 +1080,11 @@ int sh_set_schedule_two (char * str)
  * Main program
  *
  *******************************************************/
+#if !defined(SH_CUTEST)
 int main(int argc, char * argv[])
+#else
+int undef_main(int argc, char * argv[])
+#endif
 {
 #if defined(INET_SYSLOG)
   extern int    create_syslog_socket (int flag);
@@ -1092,6 +1102,8 @@ int main(int argc, char * argv[])
 
   int           flag_check_1 = 0;
   int           flag_check_2 = 0;
+
+  int           check_done   = 0;
 #endif
 
   time_t        told;
@@ -1101,7 +1113,7 @@ int main(int argc, char * argv[])
   int           res;
 
 #if defined (SH_STEALTH_NOCL)
-  char  * command_line;
+  char    command_line[256];
   int     my_argc = 0;
   char  * my_argv[32];
 #endif
@@ -1159,7 +1171,8 @@ int main(int argc, char * argv[])
       sh.flag.opts = S_TRUE;
       (void) sh_unix_setdeamon(NULL);
 #if defined (SH_WITH_CLIENT) || defined (SH_STANDALONE)
-      (void) sh_util_setchecksum(_("check"));
+      sh.flag.checkSum = SH_CHECK_CHECK;
+      /* (void) sh_util_setchecksum(_("check")); */
 #endif
       is_samhainctl_init = S_TRUE;
       sh.flag.opts = S_FALSE;
@@ -1206,12 +1219,17 @@ int main(int argc, char * argv[])
 
   /* Save the timezone.
    */
-  if ((tzptr = getenv("TZ")) != NULL)
+  if (NULL != (tzptr = getenv("TZ"))) /* flawfinder: ignore */
     {
       tzlen       = strlen(tzptr);
-       sh.timezone = malloc (tzlen + 1);
-      if (sh.timezone != NULL)
-	 (void) sl_strlcpy (sh.timezone, tzptr, tzlen + 1);
+      if (tzlen < 1024)
+	{
+	  sh.timezone = malloc (tzlen + 1);
+	  if (sh.timezone != NULL)
+	    (void) sl_strlcpy (sh.timezone, tzptr, tzlen + 1);
+	}
+      else
+	sh.timezone = NULL;
     }
   else
      sh.timezone = NULL;
@@ -1254,18 +1272,19 @@ int main(int argc, char * argv[])
   sh.flag.opts = S_TRUE;
 
 #if !defined(SH_STEALTH_NOCL)
+  sh_argc_store = argc;
+  sh_argv_store = argv;
   (void) sh_getopt_get (argc, argv);
 #else
   if (argc > 1 && argv[1] != NULL && 
       strlen(argv[1]) > 0 && strlen(NOCL_CODE) > 0)
     {
-      if ( 0 == strcmp(argv[1], NOCL_CODE) &&
-	   NULL != (command_line = (char *) SH_ALLOC(256 * sizeof(char))))
+      if ( 0 == strcmp(argv[1], NOCL_CODE) )
 	{
 	  my_argv[0] = argv[0]; ++my_argc;  
 	  command_line[0] = '\0';
-	  (void*) fgets (command_line, 255, stdin);
-	  command_line[255] = '\0';
+	  (void*) fgets (command_line, sizeof(command_line), stdin);
+	  command_line[sizeof(command_line)-1] = '\0';
 	  do {
 	    my_argv[my_argc] = 
 	      strtok( (my_argc == 1) ? command_line : NULL, " \n"); 
@@ -1275,8 +1294,11 @@ int main(int argc, char * argv[])
 	      break;
 	    }
 	  } while (my_argc < 32);
+
+	  sh_argc_store = my_argc;
+	  sh_argv_store = my_argv;
+
 	  (void) sh_getopt_get (my_argc, my_argv);
-	  SH_FREE (command_line);
 	}
       else
 	{
@@ -1426,10 +1448,7 @@ int main(int argc, char * argv[])
 #if defined(SH_WITH_SERVER) && !defined(SH_WITH_CLIENT)
 
 #if (defined(WITH_GPG) || defined(WITH_PGP))
-  /* do nothing -- we exit earlier if error 
-  if (0 != sh_gpg_check_sign (1)) 
-    aud_exit(FIL__, __LINE__, EXIT_FAILURE);
-  */
+  /* do nothing -- we exit earlier if error */
 #else
   sh_error_handle ((-1), FIL__, __LINE__, 0, MSG_START_1H,
 		   sh.prg_name, (long) sh.real.uid, 
@@ -1451,11 +1470,7 @@ int main(int argc, char * argv[])
   if (sh.flag.checkSum == SH_CHECK_CHECK) 
     {
 #if (defined(WITH_GPG) || defined(WITH_PGP))
-      /* do nothing -- we exit earlier if error 
-	 if (0 != sh_gpg_check_sign (2)) 
-	 aud_exit(FIL__, __LINE__, EXIT_FAILURE);
-      */
-      ;
+      /* do nothing -- we exit earlier if error */
 #else
       sh_error_handle ((-1), FIL__, __LINE__, 0, MSG_START_2H,
 		       sh.prg_name, (long) sh.real.uid,
@@ -1466,11 +1481,7 @@ int main(int argc, char * argv[])
   else
     {
 #if (defined(WITH_GPG) || defined(WITH_PGP))
-      /* do nothing -- we exit earlier if error 
-      if (0 != sh_gpg_check_sign (1)) 
-	aud_exit(FIL__, __LINE__, EXIT_FAILURE);
-      */
-      ;
+      /* do nothing -- we exit earlier if error */
 #else
       sh_error_handle ((-1), FIL__, __LINE__, 0, MSG_START_1H,
 		       sh.prg_name, (long) sh.real.uid,
@@ -1744,7 +1755,7 @@ int main(int argc, char * argv[])
 	      --sig_raised;
 	    }
 	  
-	  if (sig_suspend_switch == 1)  /* SIGUSR2 */
+	  if (sig_suspend_switch > 0)  /* SIGUSR2 */
 	    {
 	      TPT((0, FIL__, __LINE__, _("msg=<Suspend switch.>\n")));
 	      if (sh_global_suspend_flag == 1) {
@@ -1754,7 +1765,7 @@ int main(int argc, char * argv[])
 				sh.prg_name);
 		sh_global_suspend_flag = 1;
 	      }
-	      sig_suspend_switch = 0;
+	      --sig_suspend_switch;
 	      --sig_raised; --sig_urgent;
 	    }
 	  sig_raised = (sig_raised < 0) ? 0 : sig_raised;
@@ -1770,7 +1781,9 @@ int main(int argc, char * argv[])
       
       /* see whether its time to check files
        */
-      if      (sh.flag.checkSum == SH_CHECK_INIT)
+      if      (sh.flag.checkSum == SH_CHECK_INIT ||
+	       (sh.flag.checkSum == SH_CHECK_CHECK &&
+		(sh.flag.isdaemon == S_FALSE && sh.flag.loop == S_FALSE)))
 	{
 	  flag_check_1 = 1;
 	  if (FileSchedTwo != NULL) 
@@ -1798,6 +1811,8 @@ int main(int argc, char * argv[])
 	    }
 	}
 
+      check_done = 0;
+
       if (sh.flag.checkSum != SH_CHECK_NONE &&
 	  (flag_check_1 == 1 || flag_check_2 == 1))
 	{
@@ -1856,6 +1871,7 @@ int main(int argc, char * argv[])
 	  sh_files_reset ();
 	  flag_check_1 = 0;
 	  flag_check_2 = 0;
+	  check_done   = 1;
 
 	  (void) sh_prelink_run (NULL, NULL, 0);
 
@@ -1923,13 +1939,20 @@ int main(int argc, char * argv[])
       if      (sh.flag.checkSum == SH_CHECK_INIT)
 	sh_hash_pushdata (NULL, NULL);
 
+      /* write out database
+       */
+      if (sh.flag.checkSum == SH_CHECK_CHECK && 
+	  sh.flag.update == S_TRUE && 
+	  check_done == 1)
+	sh_hash_writeout ();
+
       /* no loop if not daemon
        */
       if (sh.flag.isdaemon != S_TRUE && sh.flag.loop == S_FALSE)
 	break; 
       if (sig_urgent > 0)
 	continue;
-      
+
       /* see whether its time to send mail
        */
 #if defined(SH_WITH_MAIL)
