@@ -107,7 +107,7 @@ int sh_html_head(SL_TICKET ticket)
 
   if (!SL_ISERROR(fd))
     {
-      while (!SL_ISERROR(status) && sh_unix_getline (fd, line, 511) > 0) 
+      while (!SL_ISERROR(status) && sh_unix_getline (fd, line, sizeof(line)) > 0) 
 	{
 	  formatted = replace_stat (line);
 	  if (formatted)
@@ -120,7 +120,7 @@ int sh_html_head(SL_TICKET ticket)
     }
   else
     {
-      qstr = sh_util_filename(DEFAULT_HTML_FILE);
+      qstr = sh_util_basename(DEFAULT_HTML_FILE);
       if (qstr != NULL)
 	{
 	  sl_snprintf(endhead, 511,
@@ -216,7 +216,7 @@ int sh_html_foot(SL_TICKET ticket)
 
   if (!SL_ISERROR(fd))
     {
-      while (!SL_ISERROR(status) && sh_unix_getline (fd, line, 511) > 0) 
+      while (!SL_ISERROR(status) && sh_unix_getline (fd, line, sizeof(line)) > 0) 
 	{
 	  status = sl_write_line (ticket, line, sl_strlen(line));
 	}
@@ -288,7 +288,7 @@ int sh_html_get_entry ()
     }
   if (!SL_ISERROR(fd))
     {
-      while (!SL_ISERROR(retval) && sh_unix_getline (fd, line, 511) > 0) 
+      while (!SL_ISERROR(retval) && sh_unix_getline (fd, line, sizeof(line)) > 0) 
 	{
 	  line_size = sl_strlen(line);
 	  add_size  = 0;
