@@ -414,9 +414,14 @@ int sh_suidchk_check_internal (char * iname)
   char          * fs;
   long            sl_status = SL_ENONE;
   struct stat     fileInfo;
+  struct stat     fileInfo_F;
+  int             file_d;
 
   file_type       theFile;
-  char            fileHash[KEY_LEN + 1];
+  char            fileHash[2*(KEY_LEN + 1)];
+
+  mode_t          umask_old;
+  int             cperm_status;
 
   SL_ENTER(_("sh_suidchk_check_internal"));
 
@@ -518,14 +523,20 @@ int sh_suidchk_check_internal (char * iname)
 	      /* fs is a STATIC string
 	       */
 	      fs = filesystem_type (tmpcat, tmpcat, &buf);
-	      if (fs != NULL && 
-		  0 != strncmp (_("nfs"),     fs, 3) && 
-		  0 != strncmp (_("proc"),    fs, 4) &&
-		  0 != strncmp (_("iso9660"), fs, 7) &&
-		  0 != strncmp (_("vfat"),    fs, 4) &&
-		  0 != strncmp (_("msdos"),   fs, 5) &&
+	      if (fs != NULL 
+#ifndef SH_SUIDTESTDIR
+		  && 
+		  0 != strncmp (_("afs"),     fs, 3) && 
 		  0 != strncmp (_("devfs"),   fs, 5) &&
-		  0 != strncmp (_("nosuid"),  fs, 6) 
+		  0 != strncmp (_("iso9660"), fs, 7) &&
+		  0 != strncmp (_("lustre"),  fs, 6) &&
+		  0 != strncmp (_("mmfs"),    fs, 4) && 
+		  0 != strncmp (_("msdos"),   fs, 5) &&
+		  0 != strncmp (_("nfs"),     fs, 3) && 
+		  0 != strncmp (_("nosuid"),  fs, 6) &&
+		  0 != strncmp (_("proc"),    fs, 4) &&
+		  0 != strncmp (_("vfat"),    fs, 4)
+#endif 
 		  )
 		{
 		  /* fprintf(stderr, "%s: %s\n", fs, tmpcat); */
@@ -563,8 +574,9 @@ int sh_suidchk_check_internal (char * iname)
 		}
 	      else
 		{
-		  if (sh.flag.checkSum == SH_CHECK_INIT  && 
-		      sh.flag.update == S_TRUE )
+		  if (sh.flag.update == S_TRUE && 
+		      (sh.flag.checkSum == SH_CHECK_INIT  || 
+		       sh.flag.checkSum == SH_CHECK_CHECK))
 		    {
 		      if (-1 == sh_hash_have_it (tmpcat))
 			{
@@ -583,7 +595,7 @@ int sh_suidchk_check_internal (char * iname)
 						 _("[SuidCheck]"), 
 						 ShSuidchkSeverity))
 			{
-			  sh_hash_pushdata (&theFile, fileHash);
+			  sh_hash_pushdata_memory (&theFile, fileHash);
 			}
 		    }
 		  else if (sh.flag.checkSum == SH_CHECK_INIT  && 
@@ -664,6 +676,8 @@ int sh_suidchk_check_internal (char * iname)
 				      }
 				    break;
 				  case SH_Q_CHANGEPERM:
+				    cperm_status = 0;
+				    file_d = -1;
 				    if (retry_lstat(FIL__, __LINE__, tmpcat, &fileInfo) == -1)
 				      {
 					status = errno;
@@ -675,8 +689,10 @@ int sh_suidchk_check_internal (char * iname)
 							 MSG_SUID_QREPORT, msg,
 							 tmp );
 					SH_FREE(msg);
+					cperm_status = -1;
 				      }
-				    else
+
+				    if (cperm_status == 0)
 				      {
 					if (0 != (caperr = sl_get_cap_qdel()))
 					  {
@@ -684,13 +700,75 @@ int sh_suidchk_check_internal (char * iname)
 							    caperr, MSG_E_SUBGEN,
 							    sh_error_message (caperr), 
 							    _("sl_get_cap_qdel"));
+					    cperm_status = -1;
 					  }
+				      }
 
-					if ((fileInfo.st_mode & S_ISUID) > 0)
-					  fileInfo.st_mode -= S_ISUID;
-					if ((fileInfo.st_mode & S_ISGID) > 0)
-					  fileInfo.st_mode -= S_ISGID;
-					if (chmod(tmpcat, fileInfo.st_mode) == -1)
+				    if (cperm_status == 0)
+				      {
+					file_d = aud_open (FIL__, __LINE__, SL_YESPRIV,
+							   tmpcat, O_RDONLY, 0);
+					if (-1 == file_d)
+					  {
+					    status = errno;
+					    msg = SH_ALLOC(SH_BUFSIZE);
+					    (void) sl_snprintf(msg, SH_BUFSIZE, _("I/O error.  errno = %ld"), status);
+					    sh_error_handle (ShSuidchkSeverity,
+							     FIL__, __LINE__, 
+							     status,
+							     MSG_SUID_QREPORT, msg,
+							     tmp );
+					    SH_FREE(msg);
+					    cperm_status = -1;
+					  }
+				      }
+
+				    if (cperm_status == 0)
+				      {
+					if (retry_fstat(FIL__, __LINE__, file_d, &fileInfo_F) == -1)
+					  {
+					    status = errno;
+					    msg = SH_ALLOC(SH_BUFSIZE);
+					    (void) sl_snprintf(msg, SH_BUFSIZE, 
+							       _("I/O error.  errno = %ld"), status);
+					    sh_error_handle (ShSuidchkSeverity,
+							     FIL__, __LINE__, 
+							     status,
+							     MSG_SUID_QREPORT, msg,
+							     tmp );
+					    SH_FREE(msg);
+					    cperm_status = -1;
+					  }
+				      }
+
+				    if (cperm_status == 0)
+				      {
+					if (fileInfo_F.st_ino  != fileInfo.st_ino ||
+					    fileInfo_F.st_dev  != fileInfo.st_dev ||
+					    fileInfo_F.st_mode != fileInfo.st_mode)
+					  {
+					    status = errno;
+					    msg = SH_ALLOC(SH_BUFSIZE);
+					    (void) sl_snprintf(msg, SH_BUFSIZE, 
+							       _("Race detected.  errno = %ld"), status);
+					    sh_error_handle (ShSuidchkSeverity,
+							     FIL__, __LINE__, 
+							     status,
+							     MSG_SUID_QREPORT, msg,
+							     tmp );
+					    SH_FREE(msg);
+					    cperm_status = -1;
+					  }
+				      }
+
+				    if ((fileInfo.st_mode & S_ISUID) > 0)
+				      fileInfo.st_mode -= S_ISUID;
+				    if ((fileInfo.st_mode & S_ISGID) > 0)
+				      fileInfo.st_mode -= S_ISGID;
+
+				    if (cperm_status == 0)
+				      {
+					if (fchmod(file_d, fileInfo.st_mode) == -1)
 					  {
 					    status = errno;
 					    msg = SH_ALLOC(SH_BUFSIZE);
@@ -711,23 +789,46 @@ int sh_suidchk_check_internal (char * iname)
 							     _("Quarantine method applied"),
 							     tmp );
 					  }
-					if (0 != (caperr = sl_drop_cap_qdel()))
+				      }
+
+				    if (0 != (caperr = sl_drop_cap_qdel()))
+				      {
+					sh_error_handle((-1), FIL__, __LINE__, 
+							caperr, MSG_E_SUBGEN,
+							sh_error_message (caperr), 
+							_("sl_drop_cap_qdel"));
+				      }
+
+				    if (file_d != -1)
+				      {
+					do {
+					  status = close (file_d);
+					} while (status == -1 && errno == EINTR);
+
+					if (-1 == status)
 					  {
-					    sh_error_handle((-1), FIL__, __LINE__, 
-							    caperr, MSG_E_SUBGEN,
-							    sh_error_message (caperr), 
-							    _("sl_drop_cap_qdel"));
+					    status = errno;
+					    msg = SH_ALLOC(SH_BUFSIZE);
+					    (void) sl_snprintf(msg, SH_BUFSIZE, 
+							       _("I/O error.  errno = %ld"), status);
+					    sh_error_handle (ShSuidchkSeverity,
+							     FIL__, __LINE__, 
+							     status,
+							     MSG_SUID_QREPORT, msg,
+							     tmp );
+					    SH_FREE(msg);
+					    cperm_status = -1;
 					  }
 				      }
 				    break;
 				  case SH_Q_MOVE:
 				    dir = SH_ALLOC(PATH_MAX+1);
 				    (void) sl_strlcpy (dir, DEFAULT_QDIR, PATH_MAX+1);
-				    if (access (dir, F_OK) != 0)
+				    if (retry_stat (FIL__, __LINE__, dir, &fileInfo) != 0)
 				      {
 					status = errno;
 					msg = SH_ALLOC(SH_BUFSIZE);
-					(void) sl_snprintf(msg, SH_BUFSIZE, _("Problem quarantining file.  File NOT quarantined.  errno = %ld (access)"), status);
+					(void) sl_snprintf(msg, SH_BUFSIZE, _("Problem quarantining file.  File NOT quarantined.  errno = %ld (stat)"), status);
 					sh_error_handle (ShSuidchkSeverity,
 							 FIL__, __LINE__, 
 							 status,
@@ -752,9 +853,7 @@ int sh_suidchk_check_internal (char * iname)
 					  }
 					else
 					  {
-					    basetmp = SH_ALLOC(1 + sl_strlen(theFile.fullpath));
-					    (void) sl_strlcpy(basetmp, theFile.fullpath, 
-						       1 + sl_strlen(theFile.fullpath));
+					    basetmp = sh_util_strdup(theFile.fullpath);
 					    filetmp = SH_ALLOC(PATH_MAX+1);
 					    (void) sl_snprintf(filetmp, PATH_MAX+1, "%s/%s", 
 							DEFAULT_QDIR, basename(basetmp));
@@ -813,6 +912,10 @@ int sh_suidchk_check_internal (char * iname)
 						(void) sl_snprintf(filetmp, PATH_MAX+1, "%s/%s.info", 
 								   DEFAULT_QDIR, 
 								   basename(theFile.fullpath));
+						/*
+						 * avoid chmod by setting umask
+						 */
+						umask_old = umask (0077);
 						filePtr = fopen (filetmp, "w+");
 						/*@-usedef@*/
 						if (filePtr)
@@ -826,23 +929,13 @@ int sh_suidchk_check_internal (char * iname)
 						    (void) fclose (filePtr);
 						  }
 						/*@+usedef@*/
+						umask (umask_old);
 					
 						sh_error_handle (ShSuidchkSeverity,
 							 	 FIL__,__LINE__,
 								 0, MSG_SUID_QREPORT,
 								 _("Quarantine method applied"),
 								 tmp );
-						if (chmod(filetmp, S_IRUSR | S_IWUSR) == -1)
-						  {
-						    status = errno;
-						    msg = SH_ALLOC(SH_BUFSIZE);
-						    (void) sl_snprintf(msg, SH_BUFSIZE, _("Problem setting permissions on quarantined file.  errno = %ld"), status);
-						    sh_error_handle (ShSuidchkSeverity,
-								     FIL__,__LINE__,
-								     status, MSG_SUID_QREPORT,
-								     msg, tmp );
-						    SH_FREE(msg);
-						  }
 					      }
 					    SH_FREE(filetmp);
 					  }
@@ -936,6 +1029,11 @@ int sh_suidchk_timer (time_t tcurrent)
   if (sh.flag.checkSum == SH_CHECK_INIT)
     return -1;
 
+  /* One-shot (not daemon and not loop forever)
+   */
+  if (sh.flag.isdaemon != S_TRUE && sh.flag.loop == S_FALSE)
+    return -1;
+
   if (ShSuidchkSched != NULL)
     {
       return test_sched(ShSuidchkSched);
@@ -969,7 +1067,11 @@ int sh_suidchk_check ()
   FileLimNum        = 0;
   FileLimTotal      = 0;
 
+#ifdef SH_SUIDTESTDIR
+  status = sh_suidchk_check_internal (SH_SUIDTESTDIR);
+#else
   status = sh_suidchk_check_internal ("/");
+#endif
 
   sh_error_handle ((-1), FIL__, __LINE__, EINVAL, MSG_SUID_SUMMARY,
 		   FileLimTotal,
@@ -1162,6 +1264,7 @@ int sh_suidchk_set_qmethod (char * c)
 {
   long val;
   int  ret = 0;
+  struct stat buf;
 
   SL_ENTER(_("sh_suidchk_set_qmethod"));
 
@@ -1183,7 +1286,7 @@ int sh_suidchk_set_qmethod (char * c)
           ShSuidchkQMethod = SH_Q_CHANGEPERM;
           break;
         case SH_Q_MOVE:
-          if (access (DEFAULT_QDIR, F_OK) != 0)
+          if (retry_stat (FIL__, __LINE__, DEFAULT_QDIR, &buf) != 0)
 	    {
 	      if (mkdir (DEFAULT_QDIR, 0750) == -1)
 		{
